CHAR = 'CharField'
FIXED_CHAR = 'FixedCharField'
TEXT = 'TextField'
UUID = 'UUIDField'
SELECT = 'SelectField'
BOOLEAN = 'BooleanField'
INTEGER = 'IntegerField'
BIG_INTEGER = 'BigIntegerField'
SMALL_INTEGER = 'SmallIntegerField'
FLOAT = 'FloatField'
NUMBER = 'NumberField'
DECIMAL = 'DecimalField'
DOUBLE_PRECISION = 'DoublePrecisionField'
MONEY = 'MoneyField'
DATE = 'DateField'
DATE_TIME = 'DateTimeField'
TIMESTAMP = 'TimestampField'
FOREIGN_KEY = 'ForeignKey'
JSON = 'JSONField'
GEOMETRY = 'GeometryField'
GEOGRAPHY = 'GeographyField'
BINARY = 'BinaryField'
